## Explain Salaries data with linear regression and ANOVA
## Accompanying file for one of the SAN solved tasks

library(carData)
library(car)
library(ggplot2)
library(dplyr)

## learn Salaries data from carData package
## further explanatory analysis of this dataset available here:
## https://rstudio-pubs-static.s3.amazonaws.com/399031_67d0656319ae4a288083cfea7a6a4200.html

# help(Salaries)

# Take a first glimpse at your data. Note which variables are numeric and which are factors
dplyr::glimpse(Salaries)
df <- Salaries

#######################
## LINEAR REGRESSION ##
#######################
# First, use sex as our variable of interest in a single variate run. Is it important in the relationship to salary?
# What insight does the coefficient give us?
lm_s <- lm(salary ~ sex, df) 
summary(lm_s)

# Next, use rank as it is likely the most relevant independent variable
# Again, what can you tell about the effect of `rank` on `salary` based on model summary?
lm_r <- lm(salary ~ rank,df) 
summary(lm_r)

# Now, use both sex and rank to fit a model. Note that both variables are categorical
# How does knowing sex help when also rank is known? We will address this issue later
lm_rs <- lm(salary ~ sex + rank, df) 
summary(lm_rs)

###########
## ANOVA ##
###########
# You saw you can use linear models to asses a relationship between categorical predictors and numeric response
# Now employ the method dedicated for such analysis - ANOVA
summary(aov(salary ~ sex, df)) 
# the same F-stat and p-value as in lm_s

# CAUTION: In ANOVA calls the ordering of independent variables matters. Compare following outputs:
summary(aov(salary ~ sex + rank, df)) 
summary(aov(salary ~ rank + sex, df)) 
# This is because ANOVA evaluates factors sequentially, i.e. how much REMAINING variance is reduced with the next predictor in sequence 
# In the second call, `sex` does not reduce much variance left after modeling salary with `rank`

## for further explanation see:
## https://towardsdatascience.com/one-of-the-most-common-unintentional-mistakes-when-running-an-anova-in-r-cfa55d332a
## https://tutorials.methodsconsultants.com/posts/the-3-different-sum-of-squares/

#######################
## ANOVA IN MODELING ##
#######################

## ANOVA is important also in estimating the importance of individual parameters in regression models (via an F-test)
anova(lm_r, lm_rs) # Compare models with `rank` and `rank+sex`. See the role of sex when added to lm

# see if adding sex to the full model is useful
anova(lm(salary ~ rank + discipline + yrs.since.phd + yrs.service, df),
      lm(salary ~ rank + discipline + yrs.since.phd + yrs.service + sex, df)) 
# This use-case is even more useful with complex models (e.g. mixed), where the comparison would not be otherwise straightforward

#######################################
## COLLINEARITY IN LINEAR REGRESSION ##
#######################################
# In this part we take a small detour to the problem of collinearity in regression, as it can be nicely demonstrated here

# Fit the full model. Is sex significant when all remaining predictors are considered?
lm_all <- lm(salary ~ rank + discipline + yrs.since.phd + yrs.service + sex, df)
summary(lm_all)

# Recall though that in the single variate analysis `sex` was significant
summary(lm_s)

# Looking at the table of rank, sex and salary, do you see the source of what you've observed?
df %>% 
  group_by(rank, sex) %>%
  summarise(n = n(),
            mean_salary = mean(salary)) %>% 
  pivot_wider(names_from = sex, values_from = n)

# You might recall from the last labs that the loss of significance occurs when two independent variables are correlated
# With categorical variables, we can check for "correlation" via the test of independence (chi-square test)
chisq.test(df$rank, df$sex)
# The test detects disbalance in the following table:
table(df$rank, df$sex)
# Note, how men are over-represented in the high-salary 'Prof' group. 
# Think of what counts would you expect if the variables were independent?

# Collinear predictors can be systematically detected 
# Variance Inflation Factors (VIF) measures how well can we predict one independent variable using the remaining ones
# A general guideline is that a VIF larger than 5 indicates that the model has problems estimating the coefficients due to high collinearity
vif(lm_all)
cor(df$yrs.service, df$yrs.since.phd) # the two variables with large VIF strongly correlate

X11()
ggplot(df, aes(x=yrs.since.phd, y=yrs.service)) + 
  geom_point()+
  geom_smooth(method=lm)
savePlot("collin_scatter.png")

lm_w_phd <- lm(salary ~ rank + discipline + yrs.since.phd + sex, df) # remove one of the two variables to resolve the collinearity issue
vif(lm_w_phd) # looks OK, no VIF>5
summary(lm_w_phd) # the model performs as well as lm_all, moreover, stability increased

lm_w_service <- lm(salary ~ rank + discipline + yrs.service + sex, df) # remove one of the two variables to resolve the collinearity issue
vif(lm_w_service) # looks OK, no VIF>5
summary(lm_w_service) # the model performs as well as lm_all, moreover, stability increased

